#include <stdio.h>
#include <stdlib.h>

#include "kernel.h"
#include "random.h"
#include "process.h"

#include "config.h"


struct RunQ *runq;

struct PhysPage *first_free_page;
struct PhysPage *first_busy_page;

size_t system_time;

// metrics
int free_pages_cnt;
int busy_pages_cnt;


int
main(void)
{
	system_time = 0;

	free_pages_cnt = 0;
	busy_pages_cnt = 0;

	first_busy_page = NULL;
	
	first_free_page = malloc(sizeof(struct PhysPage));
	first_free_page->ppn = 0;
	first_free_page->busy_flag = 0;
	first_free_page->prev = first_free_page;
	first_free_page->next = first_free_page;
	free_pages_cnt++;

	for (int i = 1; i < PHYSICAL_PAGE_AMOUNT; i++) {
		struct PhysPage *pp = malloc(sizeof(struct PhysPage));
		pp->ppn = i;
		pp->busy_flag = 0;

		/*
		 *      insert pp into here
		 *               |
		 *               v
		 *           [ ]   [ ]   [ ]
		 *                  ^
		 *                  |
		 *          *first_free_page
		 */
		pp->prev = first_free_page->prev;
		pp->next = first_free_page; 
		pp->next->prev = pp;
		pp->prev->next = pp;

		free_pages_cnt++;
	}

	runq = RunQ(10);

	for (size_t i = 0; i < 10; i++) {
		//RUNQ_add_process(1000 + randint(29000), 10 + randint(190));
		RUNQ_add_process(30000, 200, 10);
	}

	for (int i = 0; i < 10000; i++) {
		//int removal_requested = PROCESS_run_for(runq->current_proc, 50);
		int removal_requested = PROCESS_run_for(runq->current_proc, 70);

		if (removal_requested) {
			RUNQ_remove_current_process();
			RUNQ_add_process(30000, 200, 10);
		} else {
			runq->current_proc = runq->current_proc->next;
		}

#if DISABLE_LOGGING != 1
		printf("[main:metrics] Memory usage stats: busy %d, free %d, total %d\n", busy_pages_cnt, free_pages_cnt, free_pages_cnt+busy_pages_cnt);
#endif

		//KERNEL_update_job(8);
		KERNEL_update_job(16);

		if (!runq->current_proc) {
			printf("[main:scheduling] No processes left to execute!\n");
			break;
		}
	}
}
